@@ -1,8 +1,4 @@
-//! A navigation system. Use [`NavigationPlugin`] to get it working
-//!
-//! See [the RFC](https://github.com/nicopap/rfcs/blob/ui-navigation/rfcs/41-ui-navigation.md)
-//! for a deep explanation on how this works.
-// TODO: review all uses of `.unwrap()`!
+#![doc = include_str!("../Readme.md")]
 // Notes on the structure of this file:
 //
 // All "helper functions" are defined after `resolve`,
@@ -12,6 +8,7 @@ mod commands;
 #[cfg(feature = "bevy-ui")]
 pub mod components;
 mod events;
+pub mod marker;
 pub mod systems;
 
 use std::cmp::Ordering;
@@ -65,7 +62,8 @@ pub struct NavLock {
     entity: Option<Entity>,
 }
 impl NavLock {
-    /// The `Entity` that triggered the lock.
+    /// The [`Entity`](https://docs.rs/bevy/latest/bevy/ecs/entity/struct.Entity.html)
+    /// that triggered the lock.
     pub fn entity(&self) -> Option<Entity> {
         self.entity
     }
@@ -199,11 +197,7 @@ impl NavMenu {
 
     /// Set this menu as having no parents
     pub fn root() -> Self {
-        NavMenu {
-            focus_parent: None,
-            setting: MenuSetting::ClosedXY,
-            non_inert_child: CacheOption::NotYetCached,
-        }
+        Self::new(None)
     }
 
     /// Set this menu as closed (no cycling)
@@ -241,11 +235,7 @@ impl NavMenu {
     /// to reach `NavMenu` X from `Focusable` Y if there is a path from
     /// `NavMenu` X to `Focusable` Y.
     pub fn reachable_from(focusable: Entity) -> Self {
-        NavMenu {
-            focus_parent: Some(focusable),
-            setting: MenuSetting::ClosedXY,
-            non_inert_child: CacheOption::NotYetCached,
-        }
+        Self::new(Some(focusable))
     }
 
     fn with_non_inert_child(self, child: Option<Entity>) -> Self {
@@ -266,7 +256,8 @@ enum ActionType {
     Lock,
 }
 
-/// An [`Entity`] that can be navigated to using the ui navigation system.
+/// An [`Entity`](https://docs.rs/bevy/latest/bevy/ecs/entity/struct.Entity.html)
+/// that can be navigated to using the ui navigation system.
 ///
 /// It is in one of multiple [`FocusState`], you can check its state with
 /// the [`Focusable::state`] method or any of the `is_*` `Focusable` methods.
@@ -503,12 +494,11 @@ fn resolve(
             };
             let siblings = match parent {
                 Some(parent) => children_focusables(parent, queries),
-                None if !queries.focusables.is_empty() => {
-                    let focusables: Vec<_> = queries.focusables.iter().map(|tpl| tpl.0).collect();
-                    NonEmpty::try_from(focusables).unwrap()
-                }
                 None => {
-                    panic!("There must be at least one `Focusable` when sending a `NavRequest`!")
+                    let focusables: Vec<_> = queries.focusables.iter().map(|tpl| tpl.0).collect();
+                    NonEmpty::try_from(focusables).expect(
+                        "There must be at least one `Focusable` when sending a `NavRequest`!",
+                    )
                 }
             };
             let to = resolve_2d(focused, direction, cycles, &siblings, &queries.transform);
@@ -555,7 +545,7 @@ fn resolve(
                 let cycles = menu.setting.cycles();
                 let to = or_none!(resolve_scope(focused, scope_dir, cycles, &siblings));
                 let extra = match child_menu(*to, queries) {
-                    Some((_, menu)) => focus_deep(menu.clone(), queries),
+                    Some((_, menu)) => focus_deep(menu, queries),
                     None => Vec::new(),
                 };
                 let to = (extra, *to).into();
@@ -621,7 +611,12 @@ fn listen_nav_requests(
                 !matches!(err, QuerySingleError::MultipleEntities(_)),
                 "Multiple entities with Focused component, this should not happen"
             );
-            queries.focusables.iter().next().unwrap().0
+            queries
+                .focusables
+                .iter()
+                .next()
+                .expect("Before sending a NavRequest, AT LEAST ONE `Focusable` should exist")
+                .0
         });
         let event = resolve(focused_id, *request, &queries, &mut lock, Vec::new());
         // Change focus state of relevant entities
@@ -658,21 +653,18 @@ fn child_menu<'a>(focusable: Entity, queries: &'a NavQueries) -> Option<(Entity,
 
 /// The [`NavMenu`] containing `focusable`, if any
 fn parent_menu(focusable: Entity, queries: &NavQueries) -> Option<(Entity, NavMenu)> {
-    let Parent(parent) = queries.parents.get(focusable).ok()?;
-    match queries.menus.get(*parent) {
-        Ok(menu) => Some((*parent, menu.1.clone())),
-        Err(_) => parent_menu(*parent, queries),
+    let &Parent(parent) = queries.parents.get(focusable).ok()?;
+    match queries.menus.get(parent) {
+        Ok(menu) => Some((parent, menu.1.clone())),
+        Err(_) => parent_menu(parent, queries),
     }
 }
 
 /// All sibling [`Focusable`]s within a single [`NavMenu`]
 fn children_focusables(menu: Entity, queries: &NavQueries) -> NonEmpty<Entity> {
     let ret = children_focusables_helper(menu, queries);
-    assert!(
-        !ret.is_empty(),
-        "A NavMenu MUST AT LEAST HAVE ONE Focusable child, {menu:?} has none",
-    );
-    NonEmpty::try_from(ret).unwrap()
+    NonEmpty::try_from(ret)
+        .expect("A NavMenu MUST AT LEAST HAVE ONE Focusable child, {menu:?} has none")
 }
 
 fn children_focusables_helper(menu: Entity, queries: &NavQueries) -> Vec<Entity> {
@@ -721,6 +713,7 @@ fn trim_common_tail<T: PartialEq>(v1: &mut NonEmpty<T>, v2: &mut NonEmpty<T>) {
     let mut i2 = v2.len().get() - 1;
     loop {
         if v1[i1] != v2[i2] {
+            // unwraps: any usize + 1 (saturating) is NonZero
             let l1 = NonZeroUsize::new(i1.saturating_add(1)).unwrap();
             let l2 = NonZeroUsize::new(i2.saturating_add(1)).unwrap();
             v1.truncate(l1);
@@ -755,7 +748,7 @@ fn root_path(mut from: Entity, queries: &NavQueries) -> NonEmpty<Entity> {
 
 /// Navigate downward the menu hierarchy N steps, and return the path to the
 /// last level reached
-fn focus_deep(mut menu: NavMenu, queries: &NavQueries) -> Vec<Entity> {
+fn focus_deep<'a>(mut menu: &'a NavMenu, queries: &'a NavQueries) -> Vec<Entity> {
     let mut ret = Vec::with_capacity(4);
     loop {
         let last = match menu.non_inert_child() {
@@ -766,7 +759,7 @@ fn focus_deep(mut menu: NavMenu, queries: &NavQueries) -> Vec<Entity> {
             None => return ret,
         };
         menu = match child_menu(last, queries) {
-            Some((_, menu)) => menu.clone(),
+            Some((_, menu)) => menu,
             None => return ret,
         };
     }
@@ -784,14 +777,11 @@ fn resolve_index(
     max_value: usize,
 ) -> Option<usize> {
     use ScopeDirection::*;
-    match (direction, cycles, from) {
-        (Previous, true, 0) => Some(max_value),
-        (Previous, true, from) => Some(from - 1),
-        (Previous, false, 0) => None,
-        (Previous, false, from) => Some(from - 1),
-        (Next, true, from) => Some((from + 1) % (max_value + 1)),
-        (Next, false, from) if from == max_value => None,
-        (Next, false, from) => Some(from + 1),
+    match (direction, from) {
+        (Previous, 0) => cycles.then(|| max_value),
+        (Previous, from) => Some(from - 1),
+        (Next, from) if from == max_value => cycles.then(|| 0),
+        (Next, from) => Some(from + 1),
     }
 }
 
