@@ -4,7 +4,6 @@ use crate::{max_by_in_iter, Focusable, Focused};
 use bevy::ecs::system::SystemParam;
 use bevy::math::Vec3Swizzles;
 use bevy::prelude::*;
-use bevy::ui::CAMERA_UI;
 
 /// Control default ui navigation input buttons
 pub struct InputMapping {
@@ -26,6 +25,8 @@ pub struct InputMapping {
     pub previous_button: GamepadButtonType,
     /// Gamepad button for [`ScopeDirection::Next`] [`NavRequest::ScopeMove`]
     pub next_button: GamepadButtonType,
+    /// Gamepad button for [`NavRequest::Free`]
+    pub free_button: GamepadButtonType,
     /// Keyboard key for [`Direction::West`] [`NavRequest::Move`]
     pub key_left: KeyCode,
     /// Keyboard key for [`Direction::East`] [`NavRequest::Move`]
@@ -52,6 +53,8 @@ pub struct InputMapping {
     pub key_next_alt: KeyCode,
     /// Keyboard key for [`ScopeDirection::Previous`] [`NavRequest::ScopeMove`]
     pub key_previous: KeyCode,
+    /// Keyboard key for [`NavRequest::Free`]
+    pub key_free: KeyCode,
     /// Mouse button for [`NavRequest::Action`]
     pub mouse_action: MouseButton,
 }
@@ -67,6 +70,7 @@ impl Default for InputMapping {
             cancel_button: GamepadButtonType::East,
             previous_button: GamepadButtonType::LeftTrigger,
             next_button: GamepadButtonType::RightTrigger,
+            free_button: GamepadButtonType::Start,
             key_left: KeyCode::A,
             key_right: KeyCode::D,
             key_up: KeyCode::W,
@@ -80,6 +84,7 @@ impl Default for InputMapping {
             key_next: KeyCode::E,
             key_next_alt: KeyCode::Tab,
             key_previous: KeyCode::Q,
+            key_free: KeyCode::Escape,
             mouse_action: MouseButton::Left,
         }
     }
@@ -107,7 +112,7 @@ pub fn default_gamepad_input(
     mut ui_input_status: Local<bool>,
 ) {
     use Direction::*;
-    use NavRequest::{Action, Cancel, Move, ScopeMove};
+    use NavRequest::{Action, Cancel, Free, Move, ScopeMove};
 
     let pad = Gamepad(0);
     macro_rules! axis_delta {
@@ -138,6 +143,7 @@ pub fn default_gamepad_input(
         input_mapping.action_button => Action,
         input_mapping.cancel_button => Cancel,
         input_mapping.next_button => ScopeMove(ScopeDirection::Next),
+        input_mapping.free_button => Free,
         input_mapping.previous_button => ScopeMove(ScopeDirection::Previous)
     };
     for (key, request) in command_mapping {
@@ -177,6 +183,7 @@ pub fn default_keyboard_input(
         input_mapping.key_right_alt => Move(East),
         input_mapping.key_next => ScopeMove(ScopeDirection::Next),
         input_mapping.key_next_alt => ScopeMove(ScopeDirection::Next),
+        input_mapping.key_free => Free,
         input_mapping.key_previous => ScopeMove(ScopeDirection::Previous)
     };
     for (key, request) in command_mapping {
@@ -187,34 +194,38 @@ pub fn default_keyboard_input(
 }
 
 #[derive(SystemParam)]
-pub struct NodePosQuery<'w, 's> {
-    entities: Query<'w, 's, (Entity, &'static Node, &'static GlobalTransform), With<Focusable>>,
-    cam_names: Query<'w, 's, (&'static Camera, &'static GlobalTransform)>,
+pub struct NodePosQuery<'w, 's, T: Component, U: Component> {
+    entities: Query<'w, 's, (Entity, &'static T, &'static GlobalTransform), With<Focusable>>,
+    cam: Query<'w, 's, &'static GlobalTransform, With<U>>,
+}
+impl<'w, 's, T: Component, U: Component> NodePosQuery<'w, 's, T, U> {
+    fn camera_offset(&self) -> Vec2 {
+        self.cam
+            .get_single()
+            .ok()
+            .map_or(Vec2::ZERO, |trans| trans.translation.xy())
+    }
 }
 
-fn is_in_node(at: Vec2, (_, node, trans): &(Entity, &Node, &GlobalTransform)) -> bool {
+fn is_in_node<T: ScreenSize>(at: Vec2, (_, node, trans): &(Entity, &T, &GlobalTransform)) -> bool {
     let ui_pos = trans.translation.truncate();
-    let node_half_size = node.size / 2.0;
+    let node_half_size = node.size() / 2.0;
     let min = ui_pos - node_half_size;
     let max = ui_pos + node_half_size;
     (min.x..max.x).contains(&at.x) && (min.y..max.y).contains(&at.y)
 }
 
 /// Check which [`Focusable`] displays below `at` if any
-pub fn ui_focusable_at(at: Vec2, query: &NodePosQuery) -> Option<Entity> {
-    let ui_cam_name = query
-        .cam_names
-        .iter()
-        .find(|cam| cam.0.name.as_deref() == Some(CAMERA_UI));
-    let ui_camera_position = if let Some((_, trans)) = ui_cam_name {
-        trans.translation.xy()
-    } else {
-        Vec2::ZERO
-    };
+pub fn ui_focusable_at<T, U>(at: Vec2, query: &NodePosQuery<T, U>) -> Option<Entity>
+where
+    T: ScreenSize + Component,
+    U: Component,
+{
+    let ui_cam_position = query.camera_offset();
     let under_mouse = query
         .entities
         .iter()
-        .filter(|query_elem| is_in_node(at + ui_camera_position, query_elem));
+        .filter(|query_elem| is_in_node(at + ui_cam_position, query_elem));
     max_by_in_iter(under_mouse, |elem| elem.2.translation.z).map(|elem| elem.0)
 }
 
@@ -222,8 +233,77 @@ fn cursor_pos(windows: &Windows) -> Option<Vec2> {
     windows.get_primary().and_then(|w| w.cursor_position())
 }
 
+pub trait ScreenSize {
+    fn size(&self) -> Vec2;
+}
+
+#[cfg(feature = "bevy-ui")]
+impl ScreenSize for Node {
+    fn size(&self) -> Vec2 {
+        self.size
+    }
+}
+
+#[cfg(feature = "bevy-ui")]
+#[derive(Component)]
+pub struct UiCamera;
+
 /// A system to send mouse control events to the focus system
 ///
+/// Unlike [`generic_default_mouse_input`], this system is gated by the
+/// `bevy-ui` feature. It relies on bevy/render specific types:
+/// `bevy::render::Camera` and `bevy::ui::Node`.
+///
+/// Which button to press to cause an action event is specified in the
+/// [`InputMapping`] resource.
+///
+/// You may however need to customize the behavior of this system (typically
+/// when integrating in the game) in this case, you should write your own
+/// system that sends [`NavRequest`](crate::NavRequest) events. You may use
+/// [`ui_focusable_at`] to tell which focusable is currently being hovered.
+#[cfg(feature = "bevy-ui")]
+#[allow(clippy::too_many_arguments)]
+pub fn default_mouse_input(
+    input_mapping: Res<InputMapping>,
+    windows: Res<Windows>,
+    mouse: Res<Input<MouseButton>>,
+    touch: Res<Touches>,
+    focusables: NodePosQuery<Node, UiCamera>,
+    names: Query<(Entity, &Camera)>,
+    focused: Query<Entity, With<Focused>>,
+    mut cmds: Commands,
+    nav_cmds: EventWriter<NavRequest>,
+    last_pos: Local<Vec2>,
+) {
+    use bevy::ui::CAMERA_UI;
+    if focusables.cam.get_single().is_ok() {
+        generic_default_mouse_input(
+            input_mapping,
+            windows,
+            mouse,
+            touch,
+            focusables,
+            focused,
+            nav_cmds,
+            last_pos,
+        );
+    } else {
+        let ui_cam = names
+            .iter()
+            .find(|cam| cam.1.name.as_deref() == Some(CAMERA_UI));
+        if let Some((ui_cam_entity, _)) = ui_cam {
+            cmds.entity(ui_cam_entity).insert(UiCamera);
+        }
+    }
+}
+
+/// A generic system to send mouse control events to the focus system
+///
+/// `T` must be a component assigned to `Focusable` elements that implements
+/// the [`ScreenSize`] trait. `M` is simply a marker trait for your camera
+/// entity, if no entities has the `M` component, we assume the screen space
+/// is not offset from the `GlobalTransform` of focusable elements.
+///
 /// Which button to press to cause an action event is specified in the
 /// [`InputMapping`] resource.
 ///
@@ -232,29 +312,21 @@ fn cursor_pos(windows: &Windows) -> Option<Vec2> {
 /// system that sends [`NavRequest`](crate::NavRequest) events. You may use
 /// [`ui_focusable_at`] to tell which focusable is currently being hovered.
 #[allow(clippy::too_many_arguments)]
-pub fn default_mouse_input(
+pub fn generic_default_mouse_input<T: ScreenSize + Component, M: Component>(
     input_mapping: Res<InputMapping>,
     windows: Res<Windows>,
     mouse: Res<Input<MouseButton>>,
     touch: Res<Touches>,
-    focusables: NodePosQuery,
+    focusables: NodePosQuery<T, M>,
     focused: Query<Entity, With<Focused>>,
     mut nav_cmds: EventWriter<NavRequest>,
     mut last_pos: Local<Vec2>,
 ) {
-    let ui_cam_name = focusables
-        .cam_names
-        .iter()
-        .find(|cam| cam.0.name.as_deref() == Some(CAMERA_UI));
-    let ui_camera_position = if let Some((_, trans)) = ui_cam_name {
-        trans.translation.xy()
-    } else {
-        Vec2::ZERO
-    };
     let cursor_pos = match cursor_pos(&windows) {
         Some(c) => c,
         None => return,
     };
+    let ui_camera_position = focusables.camera_offset();
     let released = mouse.just_released(input_mapping.mouse_action) || touch.just_released(0);
     let focused = focused.get_single();
     // Return early if cursor didn't move since last call
